@@ -38,6 +38,17 @@
 #include "load-graph.h"
 #include "cellrenderer.h"
 
+
+static void
+kill_process_helper(ProcData *procdata, int sig)
+{
+	if (procdata->config.show_kill_warning)
+		procdialog_create_kill_dialog (procdata, sig);
+	else
+		kill_process (procdata, sig);
+}
+
+
 void
 cb_preferences_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
@@ -53,31 +64,20 @@ cb_renice (GtkMenuItem *menuitem, gpointer data)
 	ProcData * const procdata = data;
 
 	procdialog_create_renice_dialog (procdata);
-
 }
 
 
 void
 cb_end_process (GtkMenuItem *menuitem, gpointer data)
 {
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_kill_warning)
-		procdialog_create_kill_dialog (procdata, SIGTERM);
-	else
-		kill_process (procdata, SIGTERM);
+	kill_process_helper(data, SIGTERM);
 }
 
 
 void
 cb_kill_process (GtkMenuItem *menuitem, gpointer data)
 {
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_kill_warning)
-		procdialog_create_kill_dialog (procdata, SIGKILL);
-	else
-		kill_process (procdata, SIGKILL);
+	kill_process_helper(data, SIGKILL);
 }
 
 
@@ -114,55 +114,26 @@ cb_hide_process (GtkMenuItem *menuitem, gpointer data)
 void
 cb_about_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
-
-	GtkWidget *about;
-	GdkPixbuf *pixbuf;
-	GError    *error = NULL;
-	gchar     *file;
-
-
 	static const gchar *authors[] = {
-		N_("Kevin Vandersloot"),
+		"Kevin Vandersloot",
 		N_("Jorgen Scheibengruber - nicer devices treeview"),
+		N_("Benoît Dejean - maintainer"),
 		NULL
 	};
 
-	static const gchar *documenters[] = {
-		NULL
-	};
-
-	const gchar *translator_credits = _("translator_credits");
-
 	PROCMAN_GETTEXT_ARRAY_INIT(authors);
-	PROCMAN_GETTEXT_ARRAY_INIT(documenters);
 
-
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					  "procman.png", FALSE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, &error);
-
-	if (error) {
-		g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-		g_error_free (error);
-	}
-
-	g_free (file);
-
-
-	about = gnome_about_new (_("System Monitor"), VERSION,
-				 _("(C) 2001 Kevin Vandersloot"),
-				 _("System resources monitor"),
-				 authors,
-				 documenters,
-				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-
-	if (pixbuf) {
-		g_object_unref (pixbuf);
-	}
-
-
-	gtk_widget_show (about);
+	gtk_show_about_dialog (
+		NULL,
+		"name",			_("System Monitor"),
+		"comments",		_("System resources monitor"),
+		"version",		VERSION,
+		"copyright",		"(C) 2001 - 2004 Kevin Vandersloot",
+		"logo-icon-name",	"procman",
+		"authors",		authors,
+		"translator-credits",	_("translator_credits"),
+		NULL
+		);
 }
 
 
@@ -196,24 +167,6 @@ cb_app_delete (GtkWidget *window, GdkEventAny *event, gpointer data)
 }
 
 
-#if 0
-gboolean
-cb_close_simple_dialog (GnomeDialog *dialog, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	if (procdata->timeout != -1)
-		gtk_timeout_remove (procdata->timeout);
-
-	gtk_main_quit ();
-
-	return FALSE;
-
-}
-#endif
-
-
-
 void
 cb_proc_combo_changed (GtkComboBox *combo, gpointer data)
 {
@@ -230,97 +183,10 @@ cb_proc_combo_changed (GtkComboBox *combo, gpointer data)
 }
 
 
-void
-popup_menu_renice (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	procdialog_create_renice_dialog (procdata);
-}
-
-
-void
-popup_menu_show_memory_maps (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	create_memmaps_dialog (procdata);
-}
-
-
-void
-popup_menu_hide_process (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_hide_message)
-		procdialog_create_hide_dialog (procdata);
-	else
-		add_selected_to_blacklist (procdata);
-}
-
-
-void
-popup_menu_end_process (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_kill_warning)
-		procdialog_create_kill_dialog (procdata, SIGTERM);
-	else
-		kill_process (procdata, SIGTERM);
-}
-
-
-void
-popup_menu_kill_process (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_kill_warning)
-		procdialog_create_kill_dialog (procdata, SIGKILL);
-	else
-		kill_process (procdata, SIGKILL);
-
-}
-
-
-#if 0
-void
-popup_menu_about_process (GtkMenuItem *menuitem, gpointer data)
-{
-	ProcData * const procdata = data;
-	ProcInfo *info = NULL;
-	gchar *name;
-
-	if (!procdata->selected_node)
-		return;
-
-	info = e_tree_memory_node_get_data (procdata->memory,
-					    procdata->selected_node);
-	g_return_if_fail (info != NULL);
-
-	/* FIXME: this is lame. GNOME help browser sucks balls. There should be a way
-	to first check man pages, then info pages and give a nice error message if nothing
-	exists */
-	name = g_strdup_printf("man:%s", info->cmd);
-	gnome_url_show (name);
-	g_free (name);
-}
-#endif
-
-
 void
 cb_end_process_button_pressed (GtkButton *button, gpointer data)
 {
-
-	ProcData * const procdata = data;
-
-	if (procdata->config.show_kill_warning)
-		procdialog_create_kill_dialog (procdata, SIGTERM);
-	else
-		kill_process (procdata, SIGTERM);
-
+	kill_process_helper(data, SIGTERM);
 }
 
 
@@ -550,7 +416,7 @@ compare_disks (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
 	entry = g_hash_table_lookup (new_disks, old_name);
 	if (entry) {
 		glibtop_fsusage usage;
-		gchar *used, *total;
+		gchar *used, *total, *free;
 		float percentage, bused, bfree, btotal;
 
 		glibtop_get_fsusage (&usage, entry->mountdir);
@@ -558,19 +424,22 @@ compare_disks (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
 		fsusage_stats(&usage, &bused, &bfree, &btotal, &percentage);
 
 		used = gnome_vfs_format_file_size_for_display (bused);
+		free = gnome_vfs_format_file_size_for_display (bfree);
 		total = gnome_vfs_format_file_size_for_display (btotal);
 
 		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 				    4, total,
-				    5, used,
-				    6, percentage,
-				    7, btotal,
-				    8, bfree,
+				    5, free,
+				    6, used,
+				    7, percentage,
+				    8, btotal,
+				    9, bfree,
 				    -1);
 
 		g_hash_table_remove (new_disks, old_name);
 
 		g_free (used);
+		g_free (free);
 		g_free (total);
 	}
 	else {
@@ -595,9 +464,6 @@ get_icon_for_device(GnomeIconTheme *icontheme, const char *mountpoint)
 
 	monitor = gnome_vfs_get_volume_monitor();
 	volume = gnome_vfs_volume_monitor_get_volume_for_path(monitor, mountpoint);
-
-	if(!volume) return NULL;
-
 	i_type = gnome_vfs_volume_get_icon(volume);
 	gnome_vfs_volume_unref(volume);
 
@@ -631,7 +497,7 @@ add_new_disks (gpointer key, gpointer value, gpointer data)
 	GtkTreeModel * const model = data;
 
 	glibtop_fsusage usage;
-	gchar *text[5];
+	gchar *text[6];
 	GdkPixbuf *pixbuf;
 	GnomeIconTheme *icontheme;
 	GtkTreeIter row;
@@ -659,7 +525,8 @@ add_new_disks (gpointer key, gpointer value, gpointer data)
 	text[1] = g_strdup (entry->mountdir);
 	text[2] = g_strdup (entry->type);
 	text[3] = gnome_vfs_format_file_size_for_display (btotal);
-	text[4] = gnome_vfs_format_file_size_for_display (bused);
+	text[4] = gnome_vfs_format_file_size_for_display (bfree);
+	text[5] = gnome_vfs_format_file_size_for_display (bused);
 
 	gtk_tree_store_insert (GTK_TREE_STORE (model), &row, NULL, 0);
 	gtk_tree_store_set (GTK_TREE_STORE (model), &row,
@@ -669,9 +536,10 @@ add_new_disks (gpointer key, gpointer value, gpointer data)
 			    3, text[2],
 			    4, text[3],
 			    5, text[4],
-			    6, percentage,
-			    7, btotal,
-			    8, bfree,
+			    6, text[5],
+			    7, percentage,
+			    8, btotal,
+			    9, bfree,
 			    -1);
 
 	g_free (text[0]);
@@ -679,8 +547,9 @@ add_new_disks (gpointer key, gpointer value, gpointer data)
 	g_free (text[2]);
 	g_free (text[3]);
 	g_free (text[4]);
+	g_free (text[5]);
 
-	if(pixbuf) g_object_unref (pixbuf);
+	g_object_unref (pixbuf);
 	g_object_unref (icontheme);
 }
 
