@@ -39,19 +39,12 @@ void		cb_about_activate (GtkMenuItem *menuitem, gpointer user_data);
 
 void		cb_app_exit (GtkObject *object, gpointer user_data);
 gboolean	cb_app_delete (GtkWidget *window, GdkEventAny *event, gpointer data);
-#if 0
-gboolean	cb_close_simple_dialog (GnomeDialog *dialog, gpointer data);
-#endif
+
 void		cb_proc_combo_changed (GtkComboBox *combo, gpointer data);
 
 void		cb_end_process_button_pressed (GtkButton *button, gpointer data);
 void		cb_logout (GtkButton *button, gpointer data);
 
-void		popup_menu_renice (GtkMenuItem *menuitem, gpointer data);
-void		popup_menu_show_memory_maps (GtkMenuItem *menuitem, gpointer data);
-void		popup_menu_hide_process (GtkMenuItem *menuitem, gpointer data);
-void		popup_menu_end_process (GtkMenuItem *menuitem, gpointer data);
-void		popup_menu_kill_process (GtkMenuItem *menuitem, gpointer data);
 void		popup_menu_about_process (GtkMenuItem *menuitem, gpointer data);
 
 void		cb_info_button_pressed (GtkButton *button, gpointer user_data);
