@@ -16,8 +16,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
  */
-#ifndef _PROCMAN_H_
-#define _PROCMAN_H_
+#ifndef _PROCMAN_PROCMAN_H_
+#define _PROCMAN_PROCMAN_H_
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gnome.h>
@@ -58,7 +58,6 @@ struct _ProcConfig
 	GdkColor	swap_color;
 	GdkColor	bg_color;
 	GdkColor	frame_color;
-	gboolean	simple_view;
 	gint		pane_pos;
 	gint 		num_cpus;
 };
@@ -90,44 +89,61 @@ struct _LoadGraph {
     GtkWidget *cpu_labels[GLIBTOP_NCPU];
     GtkWidget *memused_label;
     GtkWidget *memtotal_label;
+    GtkWidget *mempercent_label;
     GtkWidget *swapused_label;
     GtkWidget *swaptotal_label;
+    GtkWidget *swappercent_label;
     GdkPixmap *pixmap;
     GdkGC *gc;
     int timer_index;
     
     gboolean draw;
 
-    long cpu_time [GLIBTOP_NCPU] [NCPUSTATES];
-    long cpu_last [GLIBTOP_NCPU] [NCPUSTATES];
-    int cpu_initialized;       
+    guint64 cpu_time [GLIBTOP_NCPU] [NCPUSTATES];
+    guint64 cpu_last [GLIBTOP_NCPU] [NCPUSTATES];
+    gboolean cpu_initialized;       
+};
+
+enum
+{
+	NEEDS_REMOVAL,
+	NEEDS_ADDITION,
+	NEEDS_NOTHING
 };
 
 struct _ProcInfo
 {
-	GtkTreeIter 	node;
-	GtkTreeIter 	parent_node;
-	gboolean	visible;
-	gboolean	has_parent;
+	GtkTreeIter	node;
+	GtkTreePath	*path;
+	ProcInfo	*parent;
+	GList		*children;
+
 	GdkPixbuf	*pixbuf;
 	gchar		*name;
 	gchar		*user;
 	gchar		*arguments;
-	gint		mem;
-	gint		cpu;
-	gint		pid;
-	gint		parent_pid;
-	gint		cpu_time_last;
+
+	gchar		*status; /* shared, don't free it ! */
+	gchar		*security_context;
+
+	guint64		cpu_time_last;
+
+	guint64		mem;
+	guint64		vmsize;
+	guint64		memres;
+	guint64		memshared;
+	guint64		memrss;
+	guint64		memxserver;
+
+	guint		pcpu; /* 0% - 100% */
 	gint		nice;
-	gint		vmsize;
-	gint		memres;
-	gint		memshared;
-	gint		memrss;
-        gint            memxserver;
-	gchar		*status;
-	gboolean	running;
-	gboolean	is_thread;
-	gboolean	is_blacklisted;
+
+	guint		pid;
+
+	guint		is_visible	: 1;
+	guint		is_running	: 1;
+	guint		is_thread	: 1;
+	guint		is_blacklisted	: 1;
 };
 
 struct _ProcData
@@ -143,15 +159,42 @@ struct _ProcData
 	GtkTreeSelection *selection;
 	gint		timeout;
 	gint		disk_timeout;
+
+/*
+   'info' is GList, which has very slow lookup. On each update, glibtop
+   retrieves the full system pid list. To update the table,
+
+   foreach pid in glibtop pid list:
+	- if there's not ProcInfo with pid, add a new ProcInfo to 'info'
+	- else, update the current ProcInfo
+
+   which is very inefficient, because if a process is running at time t,
+   it's very likely to be running at t+1.
+   So if is length('info') = N, N lookups are performed on each update.
+   Average lookup iterates over N / 2 ProcInfo to find that a pid already
+   has a ProcInfo.
+   -> cost = (N x N) / 2
+
+   With 200 processes, an update costs about 20000 g_list_next(), which
+   is huge because we only want to know if we have this pid or not.
+
+   So 'pids' is a hastable to perform fast lookups.
+
+   TODO: 'info' and 'pids' could be merged
+*/
+
 	GList		*info;
+	GHashTable	*pids;
+
 	PrettyTable	*pretty_table;
 	GList		*blacklist;
 	gint		blacklist_num;
 	GConfClient	*client;
+	GtkWidget	*app;
 };
 
 void		procman_save_config (ProcData *data);
 void		procman_save_tree_state (GConfClient *client, GtkWidget *tree, gchar *prefix);
 gboolean	procman_get_tree_state (GConfClient *client, GtkWidget *tree, gchar *prefix);
 
-#endif
+#endif /* _PROCMAN_PROCMAN_H_ */
