@@ -48,12 +48,13 @@
 #include "favorites.h"
 
 
-
-static gint total_time = 0;
-static gint total_time_last = 0;
+#ifdef ENABLE_SELINUX
+#include <selinux/selinux.h>
+#endif
 
 
-#define PROCMAN_CMP(X, Y) (((X) == (Y)) ? 0 : (((X) < (Y)) ? -1 : 1))
+static guint64 total_time = 1;
+static guint64 total_time_last = 1;
 
 
 static gint
@@ -71,15 +72,6 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 	case COL_MEM:
 		return PROCMAN_CMP(infoa->mem, infob->mem);
 
-	case COL_CPU:
-		return PROCMAN_CMP(infoa->pcpu, infob->pcpu);
-
-	case COL_PID:
-		return PROCMAN_CMP(infoa->pid, infob->pid);
-
-	case COL_NICE:
-		return PROCMAN_CMP(infoa->nice, infob->nice);
-
 	case COL_VMSIZE:
 		return PROCMAN_CMP(infoa->vmsize, infob->vmsize);
 
@@ -96,12 +88,66 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 		return PROCMAN_CMP(infoa->memxserver, infob->memxserver);
 
 	default:
+		g_assert_not_reached();
 		return 0;
 	}
 }
 
 
 
+static gboolean
+can_show_security_context_column (void)
+{
+#ifdef ENABLE_SELINUX
+	switch (is_selinux_enabled()) {
+	case 1: /* We're running on an SELinux kernel */ 
+		return TRUE;
+
+	default:
+	case -1: 
+		/* Error; hide the security context column */
+
+	case 0: 
+		/* We're not running on an SELinux kernel: hide the security context column */
+		return FALSE;
+	}
+#else
+	/* Support disabled; hide the security context column */
+	return FALSE;
+#endif
+}
+
+static void
+get_process_selinux_context (ProcInfo *info)
+{
+#ifdef ENABLE_SELINUX
+	/* Directly grab the SELinux security context: */
+	security_context_t con;
+
+	if (!getpidcon (info->pid, &con)) {
+		info->security_context = g_strdup (con);
+		freecon (con);
+	}
+#endif
+}
+
+
+static void
+set_proctree_reorderable(ProcData *procdata)
+{
+	GList *columns, *col;
+	GtkTreeView *proctree;
+
+	proctree = GTK_TREE_VIEW(procdata->tree);
+
+	columns = gtk_tree_view_get_columns (proctree);
+
+	for(col = columns; col; col = col->next)
+		gtk_tree_view_column_set_reorderable(col->data, TRUE);
+
+	g_list_free(columns);
+}
+
 
 GtkWidget *
 proctable_new (ProcData * const procdata)
@@ -115,7 +161,6 @@ proctable_new (ProcData * const procdata)
 
 	static const gchar *titles[] = {
 		N_("Process Name"),
-		N_("Arguments"),
 		N_("User"),
 		N_("Status"),
 		N_("Memory"),
@@ -127,6 +172,8 @@ proctable_new (ProcData * const procdata)
 		/* xgettext:no-c-format */ N_("% CPU"),
 		N_("Nice"),
 		N_("ID"),
+		N_("Security Context"),
+		N_("Arguments"),
 		NULL,
 		"POINTER"
 	};
@@ -140,12 +187,24 @@ proctable_new (ProcData * const procdata)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 
-	model = gtk_tree_store_new (NUM_COLUMNS,  G_TYPE_STRING,
-				    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-				    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-				    G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-				    G_TYPE_INT, G_TYPE_INT, G_TYPE_INT,
-				    GDK_TYPE_PIXBUF, G_TYPE_POINTER);
+	model = gtk_tree_store_new (NUM_COLUMNS,
+				    G_TYPE_STRING,	/* Process Name */
+				    G_TYPE_STRING,	/* User		*/
+				    G_TYPE_STRING,	/* Status	*/
+				    G_TYPE_STRING,      /* Memory	*/
+				    G_TYPE_STRING,	/* VM Size	*/
+				    G_TYPE_STRING,	/* Resident Memory */
+				    G_TYPE_STRING,	/* Shared Memory */
+				    G_TYPE_STRING,	/* RSS Memory	*/
+				    G_TYPE_STRING,	/* X Server Memory */
+				    G_TYPE_UINT,	/* % CPU	*/
+				    G_TYPE_INT,		/* Nice		*/
+				    G_TYPE_UINT,	/* ID		*/
+				    G_TYPE_STRING,	/* Security Context */
+				    G_TYPE_STRING,	/* Arguments	*/
+				    GDK_TYPE_PIXBUF,	/* Icon		*/
+				    G_TYPE_POINTER	/* ProcInfo	*/
+		);
 
 	proctree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (proctree), TRUE);
@@ -174,8 +233,26 @@ proctable_new (ProcData * const procdata)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (proctree), column);
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
+
 	for (i = 1; i < NUM_COLUMNS - 2; i++) {
 		cell_renderer = gtk_cell_renderer_text_new ();
+
+		switch(i)
+		{
+		case COL_MEM:
+		case COL_VMSIZE:
+		case COL_MEMRES:
+		case COL_MEMSHARED:
+		case COL_MEMRSS:
+		case COL_MEMXSERVER:
+		case COL_CPU:
+		case COL_NICE:
+		case COL_PID:
+			g_object_set(G_OBJECT(cell_renderer),
+				     "xalign", 1.0f,
+				     NULL);
+		}
+
 		column = gtk_tree_view_column_new ();
 		gtk_tree_view_column_pack_start (column, cell_renderer, FALSE);
 		gtk_tree_view_column_set_attributes (column, cell_renderer,
@@ -195,21 +272,6 @@ proctable_new (ProcData * const procdata)
 					 sort_ints,
 					 GINT_TO_POINTER (COL_MEM),
 					 NULL);
-	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
-					 COL_CPU,
-					 sort_ints,
-					 GINT_TO_POINTER (COL_CPU),
-					 NULL);
-	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
-					 COL_PID,
-					 sort_ints,
-					 GINT_TO_POINTER (COL_PID),
-					 NULL);
-	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
-					 COL_NICE,
-					 sort_ints,
-					 GINT_TO_POINTER (COL_NICE),
-					 NULL);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
 					 COL_VMSIZE,
 					 sort_ints,
@@ -235,10 +297,20 @@ proctable_new (ProcData * const procdata)
 					 sort_ints,
 					 GINT_TO_POINTER (COL_MEMXSERVER),
 					 NULL);
+
 	procdata->tree = proctree;
 
+	set_proctree_reorderable(procdata);
+
 	procman_get_tree_state (procdata->client, proctree, "/apps/procman/proctree");
 
+	/* Override column settings by hiding this column if it's meaningless: */
+	if (!can_show_security_context_column ()) {
+		GtkTreeViewColumn *column;
+		column = gtk_tree_view_get_column (GTK_TREE_VIEW (proctree), COL_SECURITYCONTEXT);
+		gtk_tree_view_column_set_visible (column, FALSE);
+	}
+
 	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (proctree))),
 			  "changed",
 			  G_CALLBACK (cb_row_selected), procdata);
@@ -256,11 +328,12 @@ proctable_new (ProcData * const procdata)
 static void
 proctable_free_info (ProcInfo *info)
 {
-	if (!info) return;
+	g_return_if_fail(info != NULL);
+
 	g_free (info->name);
 	g_free (info->arguments);
 	g_free (info->user);
-	g_free (info->status);
+	g_free (info->security_context);
 	g_list_free (info->children);
 	g_free (info);
 }
@@ -269,30 +342,39 @@ proctable_free_info (ProcInfo *info)
 static void
 get_process_status (ProcInfo *info, const glibtop_proc_state *buf)
 {
-	g_free (info->status);
 
 	switch(buf->state)
 	{
 	case GLIBTOP_PROCESS_RUNNING:
-		info->status = g_strdup (_("Running"));
-		info->running = TRUE;
+		info->status = _("Running");
+		info->is_running = TRUE;
 		break;
 
 	case GLIBTOP_PROCESS_STOPPED:
-		info->status = g_strdup (_("Stopped"));
-		info->running = FALSE;
+		info->status = _("Stopped");
+		info->is_running = FALSE;
+		break;
+
+	case GLIBTOP_PROCESS_ZOMBIE:
+		info->status = _("Zombie");
+		info->is_running = FALSE;
+		break;
+
+	case GLIBTOP_PROCESS_UNINTERRUPTIBLE:
+		info->status = _("Uninterruptible");
+		info->is_running = FALSE;
 		break;
 
 	default:
-		info->status = g_strdup (_("Sleeping"));
-		info->running = FALSE;
+		info->status = _("Sleeping");
+		info->is_running = FALSE;
 		break;
 	}
 }
 
 
 static void
-get_process_name (ProcData *procdata, ProcInfo *info,
+get_process_name (ProcInfo *info,
 		  const gchar *cmd, const gchar *args)
 {
 
@@ -306,51 +388,42 @@ get_process_name (ProcData *procdata, ProcInfo *info,
 }
 
 
-ProcInfo *
-proctable_find_process (gint pid, gchar *name, ProcData *procdata)
+static void
+get_process_memory_info(ProcInfo *info)
 {
+	glibtop_proc_mem procmem;
+	WnckResourceUsage xresources;
 
-	GList *list = procdata->info;
+	wnck_pid_read_resource_usage (gdk_screen_get_display (gdk_screen_get_default ()),
+				      info->pid,
+				      &xresources);
 
-	while (list) {
-		ProcInfo *info = list->data;
+	glibtop_get_proc_mem(&procmem, info->pid);
 
-		if (pid == info->pid)
-			return info;
+	info->mem	= procmem.size;
+	info->vmsize	= procmem.vsize;
+	info->memres	= procmem.resident;
+	info->memshared	= procmem.share;
+	info->memrss	= procmem.rss;
 
-		if (name) {
-			if (g_strcasecmp (name, info->name) == 0)
-				return info;
-		}
+	info->memxserver = xresources.total_bytes_estimate;
+}
 
-		list = g_list_next (list);
-	}
-
-	return NULL;
+ProcInfo *
+proctable_find_process (guint pid, ProcData *procdata)
+{
+	return g_hash_table_lookup(procdata->pids, GINT_TO_POINTER(pid));
 }
 
 
 static ProcInfo *
-find_parent (ProcData *data, gint pid)
+find_parent (ProcData *procdata, guint pid)
 {
-	GList *list = data->info;
-
-
 	/* ignore init as a parent process */
 	if (pid <= 1)
 		return NULL;
 
-	while (list)
-	{
-		ProcInfo *info = list->data;
-
-		if (pid == info->pid)
-			return info;
-
-		list = g_list_next (list);
-	}
-
-	return NULL;
+	return proctable_find_process(pid, procdata);
 }
 
 
@@ -364,7 +437,7 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 
 	/* Don't show process if it is not running */
 	if ((procdata->config.whose_process == ACTIVE_PROCESSES) &&
-	    (!info->running))
+	    (!info->is_running))
 		return;
 
 	/* Don't show processes that user has blacklisted */
@@ -384,7 +457,7 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 		name = g_strdup (info->name);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
-	if (info->parent && procdata->config.show_tree && info->parent->visible) {
+	if (info->parent && procdata->config.show_tree && info->parent->is_visible) {
 		GtkTreePath *parent_node = gtk_tree_model_get_path (model, &info->parent->node);
 
 		gtk_tree_store_insert (GTK_TREE_STORE (model), &row, &info->parent->node, 0);
@@ -426,6 +499,7 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 			    COL_CPU, info->pcpu,
 			    COL_PID, info->pid,
 			    COL_NICE, info->nice,
+			    COL_SECURITYCONTEXT, info->security_context,
 			    -1);
 	g_free (mem);
 	g_free (vmsize);
@@ -436,7 +510,7 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 	g_free (name);
 
 	info->node = row;
-	info->visible = TRUE;
+	info->is_visible = TRUE;
 }
 
 
@@ -445,21 +519,21 @@ insert_info_to_tree (ProcInfo *info, ProcData *procdata)
 */
 static void
 remove_children_from_tree (ProcData *procdata, GtkTreeModel *model,
-			   GtkTreeIter *parent)
+			   GtkTreeIter *parent, GPtrArray *addition_list)
 {
 	do {
 		ProcInfo *child_info;
 		GtkTreeIter child;
 
 		if (gtk_tree_model_iter_children (model, &child, parent))
-			remove_children_from_tree (procdata, model, &child);
+			remove_children_from_tree (procdata, model, &child, addition_list);
 
 		gtk_tree_model_get (model, parent, COL_POINTER, &child_info, -1);
 		if (child_info) {
 			if (procdata->selected_process == child_info)
 				procdata->selected_process = NULL;
-			child_info ->queue = NEEDS_ADDITION;
-			child_info->visible = FALSE;
+			g_ptr_array_add(addition_list, child_info);
+			child_info->is_visible = FALSE;
 		}
 	} while (gtk_tree_model_iter_next (model, parent));
 }
@@ -473,7 +547,7 @@ remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 
 	g_return_if_fail (info);
 
-	if (!info->visible)
+	if (!info->is_visible)
 		return;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
@@ -484,7 +558,7 @@ remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 
 	gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
 
-	info->visible = FALSE;
+	info->is_visible = FALSE;
 }
 
 
@@ -512,81 +586,60 @@ remove_info_from_list (ProcInfo *info, ProcData *procdata)
 
 	/* Remove from list */
 	procdata->info = g_list_remove (procdata->info, info);
+	g_hash_table_remove(procdata->pids, GINT_TO_POINTER(info->pid));
 	proctable_free_info (info);
 }
 
 
 static void
-update_info (ProcData *procdata, ProcInfo *info, gint pid)
+update_info (ProcData *procdata, ProcInfo *info)
 {
-	GtkTreeModel *model;
-	GtkTreePath *path;
 	glibtop_proc_state procstate;
-	glibtop_proc_mem procmem;
-	glibtop_proc_uid procuid;
-	glibtop_proc_time proctime;
-	gchar *mem, *vmsize, *memres, *memshared, *memrss, *memxserver;
-	guint64 newcputime;
-	guint8 pcpu;
-	WnckResourceUsage xresources;
-
-	glibtop_get_proc_state (&procstate, pid);
-	glibtop_get_proc_mem (&procmem, pid);
-	glibtop_get_proc_uid (&procuid, pid);
-	glibtop_get_proc_time (&proctime, pid);
-	newcputime = proctime.utime + proctime.stime;
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
-
-	wnck_pid_read_resource_usage (gdk_screen_get_display (gdk_screen_get_default ()),
-				      pid,
-				      &xresources);
-
-	info->mem = procmem.size;
-	info->vmsize = procmem.vsize;
-	info->memres = procmem.resident;
-	info->memshared = procmem.share;
-	info->memrss = procmem.rss;
-	info->memxserver = xresources.total_bytes_estimate;
-
-	/* Possible enhancement - also SUBTRACT this value
-	 * from the memory of the X server itself.
-	 */
-	info->mem += info->memxserver;
-
-	if (total_time != 0)
-		pcpu = ( newcputime - info->cpu_time_last ) * 100 / total_time;
-	else
-		pcpu = 0;
-
-	pcpu = MIN(pcpu, 100);
-
-	info->cpu_time_last = newcputime;
-	info->pcpu = pcpu;
-	info->nice = procuid.nice;
 
+	glibtop_get_proc_state (&procstate, info->pid);
 	get_process_status (info, &procstate);
 
 	if (procdata->config.whose_process == ACTIVE_PROCESSES)	{
 
 		/* process started running */
-		if (info->running && (!info->visible)) {
+		if (info->is_running && (!info->is_visible)) {
 			insert_info_to_tree (info, procdata);
 			return;
 		}
 		/* process was running but not anymore */
-		else if ((!info->running) && info->visible) {
+		else if ((!info->is_running) && info->is_visible) {
 			remove_info_from_tree (info, procdata);
 			return;
 		}
-		else if (!info->running)
+		else if (!info->is_running)
 			return;
 	}
 
-	if (info->visible) {
+	if (info->is_visible) {
 		GdkRectangle rect, vis_rect;
+		GtkTreePath *path;
+		gchar *mem, *vmsize, *memres, *memshared, *memrss, *memxserver;
+
+		GtkTreeModel *model;
+		glibtop_proc_uid procuid;
+		glibtop_proc_time proctime;
+
+		glibtop_get_proc_uid (&procuid, info->pid);
+		glibtop_get_proc_time (&proctime, info->pid);
+
+		get_process_memory_info(info);
+
+		info->pcpu = (proctime.rtime - info->cpu_time_last) * 100 / total_time;
+		info->pcpu = MIN(info->pcpu, 100);
+
+		info->cpu_time_last = proctime.rtime;
+		info->nice = procuid.nice;
+
+
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
 		path = gtk_tree_model_get_path (model, &info->node);
 		gtk_tree_view_get_cell_area (GTK_TREE_VIEW (procdata->tree),
-					     path, 	NULL, &rect);
+					     path, NULL, &rect);
 		gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (procdata->tree),
 						&vis_rect);
 		gtk_tree_path_free (path);
@@ -614,6 +667,7 @@ update_info (ProcData *procdata, ProcInfo *info, gint pid)
 				    COL_CPU, info->pcpu,
 				    COL_NICE, info->nice,
 				    -1);
+		/* We don't bother updating COL_SECURITYCONTEXT as it can never change */
 		g_free (mem);
 		g_free (vmsize);
 		g_free (memres);
@@ -630,36 +684,32 @@ get_info (ProcData *procdata, gint pid)
 	ProcInfo *info = g_new0 (ProcInfo, 1);
 	glibtop_proc_state procstate;
 	glibtop_proc_time proctime;
-	glibtop_proc_mem procmem;
 	glibtop_proc_uid procuid;
 	glibtop_proc_args procargs;
 	struct passwd *pwd;
 	gchar *arguments;
-	guint64 newcputime, i;
-	WnckResourceUsage xresources;
+
+	info->pid = pid;
 
 	glibtop_get_proc_state (&procstate, pid);
 	pwd = getpwuid (procstate.uid);
 	glibtop_get_proc_time (&proctime, pid);
-	glibtop_get_proc_mem (&procmem, pid);
 	glibtop_get_proc_uid (&procuid, pid);
 	glibtop_get_proc_time (&proctime, pid);
-	newcputime = proctime.utime + proctime.stime;
-
-	wnck_pid_read_resource_usage (gdk_screen_get_display (gdk_screen_get_default ()),
-				      pid,
-				      &xresources);
 
 	arguments = glibtop_get_proc_args (&procargs, pid, 0);
-	get_process_name (procdata, info, procstate.cmd, arguments);
+	get_process_name (info, procstate.cmd, arguments);
 
 	if (arguments)
 	{
+		guint i;
+
 		for (i = 0; i < procargs.size; i++)
 		{
 			if (!arguments[i])
 				arguments[i] = ' ';
 		}
+		/* steals arguments */
 		info->arguments = arguments;
 	}
 	else
@@ -670,28 +720,17 @@ get_info (ProcData *procdata, gint pid)
 	else
 		info->user = NULL;
 
-	info->mem = procmem.size;
-	info->vmsize = procmem.vsize;
-	info->memres = procmem.resident;
-	info->memshared = procmem.share;
-	info->memrss = procmem.rss;
-	info->memxserver = xresources.total_bytes_estimate;
-
-	/* Possible enhancement - also SUBTRACT this value
-	 * from the memory of the X server itself.
-	 */
-	info->mem += info->memxserver;
-
 	info->pcpu = 0;
-	info->pid = pid;
-	info->parent_pid = procuid.ppid;
-	info->cpu_time_last = newcputime;
+	info->cpu_time_last = proctime.rtime;
 	info->nice = procuid.nice;
+
+	get_process_memory_info(info);
 	get_process_status (info, &procstate);
+	get_process_selinux_context (info);
 
 	info->pixbuf = pretty_table_get_icon (procdata->pretty_table, info->name, pid);
 
-	info->parent = find_parent (procdata, info->parent_pid);
+	info->parent = find_parent (procdata, procuid.ppid);
 	if (info->parent) {
 		info->parent->children = g_list_prepend (info->parent->children, info);
 		if(g_strcasecmp (info->name, info->parent->name) == 0
@@ -713,7 +752,7 @@ get_info (ProcData *procdata, gint pid)
 		else
 			info->is_thread = FALSE;
 
-		if (parentinfo->visible) {
+		if (parentinfo->is_visible) {
 			info->parent_node = parentinfo->node;
 			info->has_parent = TRUE;
 		}
@@ -726,30 +765,19 @@ get_info (ProcData *procdata, gint pid)
 	}
 #endif
 
-	info->visible = FALSE;
+	info->is_visible = FALSE;
 
 	return info;
 }
 
 
-static gboolean
-find_match_in_new_list (gint pid, const unsigned *pid_list, guint n)
-{
-	guint i;
-
-	for (i=0; i<n; i++){
-		if (pid_list[i] == pid) return TRUE;
-	}
-
-	return FALSE;
-}
-
 
 static void
-refresh_list (ProcData *data, const unsigned *pid_list, guint n)
+refresh_list (ProcData *procdata, const unsigned *pid_list, const guint n)
 {
-	ProcData *procdata = data;
+	GHashTable *pid_hash;
 	GList *list;
+	GPtrArray *addition_list = g_ptr_array_new ();
 	GPtrArray *removal_list = g_ptr_array_new ();
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
 	guint i;
@@ -758,55 +786,57 @@ refresh_list (ProcData *data, const unsigned *pid_list, guint n)
 	for(i = 0; i < n; ++i) {
 		ProcInfo *info;
 
-		info = proctable_find_process (pid_list[i], NULL, procdata);
+		info = proctable_find_process (pid_list[i], procdata);
 		if (!info) {
 			info = get_info (procdata, pid_list[i]);
-			info->queue = NEEDS_ADDITION;
+			g_ptr_array_add(addition_list, info);
 			procdata->info = g_list_append (procdata->info, info);
+			g_hash_table_insert(procdata->pids, GINT_TO_POINTER(info->pid), info);
 		}
 		else {
-			update_info (procdata, info, info->pid);
+			update_info (procdata, info);
 		}
 	}
 
+
 	/* Remove processes */
-	list = procdata->info;
-	while (list) {
+
+	/* use a hash for fast lookup
+	   !NULL as data for every key */
+
+	pid_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
+	for(i = 0; i < n; ++i)
+		g_hash_table_insert(pid_hash,
+				    GINT_TO_POINTER(pid_list[i]),
+				    GINT_TO_POINTER(!NULL));
+
+
+	for(list = procdata->info; list; list = g_list_next (list)) {
 		ProcInfo * const info = list->data;
 
-		if (!find_match_in_new_list (info->pid, pid_list, n))  {
+		if(!g_hash_table_lookup(pid_hash, GINT_TO_POINTER(info->pid)))
+		{
 			g_ptr_array_add (removal_list, info);
-			info->queue = NEEDS_REMOVAL;
 			/* remove all children from tree */
-			if (info->visible) {
+			if (info->is_visible) {
 				GtkTreeIter child;
 				if (gtk_tree_model_iter_children (model, &child, &info->node))
-					remove_children_from_tree (procdata, model, &child);
+					remove_children_from_tree (procdata, model, &child, addition_list);
 			}
 		}
-		list = g_list_next (list);
 	}
 
+	g_hash_table_destroy(pid_hash);
+
+
 	/* Add or remove processes from the tree */
-	list = procdata->info;
-	while (list) {
-		ProcInfo * const info = list->data;
-
-		if (info->queue == NEEDS_REMOVAL) {
-			remove_info_from_tree (info, procdata);
-			info->queue = NEEDS_NOTHING;
-		} else if (info->queue == NEEDS_ADDITION) {
-			insert_info_to_tree (info, procdata);
-			info->queue = NEEDS_NOTHING;
-		}
-
-		list = g_list_next (list);
-	}
+	g_ptr_array_foreach(removal_list,  (GFunc) remove_info_from_tree, procdata);
+	g_ptr_array_foreach(addition_list, (GFunc) insert_info_to_tree  , procdata);
 
 	/* Remove processes from the internal GList */
-	for(i = 0; i < removal_list->len; ++i)
-		remove_info_from_list (removal_list->pdata[i], procdata);
+	g_ptr_array_foreach(removal_list,  (GFunc) remove_info_from_list, procdata);
 
+	g_ptr_array_free (addition_list, TRUE);
 	g_ptr_array_free (removal_list, TRUE);
 }
 
@@ -842,6 +872,8 @@ proctable_update_list (ProcData * const procdata)
 	refresh_list (procdata, pid_list, proclist.number);
 
 	g_free (pid_list);
+
+	/* proclist.number == g_list_length(procdata->info) == g_hash_table_size(procdata->pids) */
 }
 
 
@@ -884,6 +916,9 @@ proctable_free_table (ProcData * const procdata)
 
 	g_list_free (procdata->info);
 	procdata->info = NULL;
+
+	g_hash_table_destroy(procdata->pids);
+	procdata->pids = g_hash_table_new(g_direct_hash, g_direct_equal);
 }
 
 
@@ -893,7 +928,6 @@ proctable_search_table (ProcData *procdata, gchar *string)
 	GList *list = procdata->info;
 	GtkWidget *dialog;
 	GtkTreeModel *model;
-	gchar *error;
 	static gint increment = 0;
 	gint index;
 	static gchar *last = NULL;
@@ -918,7 +952,7 @@ proctable_search_table (ProcData *procdata, gchar *string)
 	while (list)
 	{
 		ProcInfo *info = list->data;
-		if (strstr (info->name, string) && info->visible)
+		if (strstr (info->name, string) && info->is_visible)
 		{
 			GtkTreePath *node = gtk_tree_model_get_path (model, &info->node);
 
@@ -935,7 +969,7 @@ proctable_search_table (ProcData *procdata, gchar *string)
 			gtk_tree_path_free (node);
 		}
 
-		if (info->user && strstr (info->user, string) && info->visible)
+		if (info->user && strstr (info->user, string) && info->is_visible)
 		{
 			GtkTreePath *node = gtk_tree_model_get_path (model, &info->node);
 
@@ -956,13 +990,12 @@ proctable_search_table (ProcData *procdata, gchar *string)
 	}
 
 	if (index == increment) {
-		error = g_strdup_printf (_("%s could not be found."), string);
 		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						 "%s", error, NULL);
+						 _("%s could not be found."), string,
+						 NULL);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
-		g_free (error);
 	}
 	else {
 		/* no more cases found. Start the search anew */
